/*!
 * mongoose-devise v1.0.1
 * (c) 2018-present Carvalho, Vinicius Luiz <carvalho.viniciusluiz@gmail.com>
 * Released under the MIT License.
 */
import mongoose from 'mongoose';
import crypto from 'crypto';
import moment from 'moment';
import _assertPlus from 'assert-plus';
import bcryptjs from 'bcryptjs';
import os from 'os';

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var email = createCommonjsModule(function (module, exports) {



var _mongoose2 = _interopRequireDefault(mongoose);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var regex = /^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/;

function isEmail(val, options) {
  var required = typeof options.required === 'function' ? options.required() : options.required;
  var passedAllowBlank = options.allowBlank && (val === '' || val === null);

  if (passedAllowBlank && !required) {
    return true;
  }

  return regex.test(val);
}

exports.isEmail = isEmail;

function Email(path, options) {
  _mongoose2.default.SchemaTypes.String.call(this, path, options);

  function isValid(val) {
    return isEmail(val, options);
  }

  this.validate(isValid, options.message || 'invalid email address');
}

Object.setPrototypeOf(Email.prototype, _mongoose2.default.SchemaTypes.String.prototype);
_mongoose2.default.Schema.Types.Email = Email;
_mongoose2.default.Types.Email = String;
module.exports = Email;
});

unwrapExports(email);
var email_1 = email.isEmail;

var errors = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});

var DeviseError = (function (Error) {
  function DeviseError(message) {
    Error.call(this, message);
    this.name = 'DeviseError';
    this.stack = new Error().stack;
    Object.setPrototypeOf(this, DeviseError.prototype);
  }

  if ( Error ) DeviseError.__proto__ = Error;
  DeviseError.prototype = Object.create( Error && Error.prototype );
  DeviseError.prototype.constructor = DeviseError;

  return DeviseError;
}(Error));

exports.DeviseError = DeviseError;
});

unwrapExports(errors);
var errors_1 = errors.DeviseError;

var tokenizer = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.genToken = exports.Tokenizer = undefined;



var _crypto2 = _interopRequireDefault(crypto);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var Tokenizer = function Tokenizer(secret) {
  this.cipher = undefined;
  this.decipher = undefined;

  this.encrypt = function (text) {
    var crypted = this.cipher.update(text, 'utf8', 'hex');
    crypted += this.cipher.final('hex');
    return crypted;
  };

  this.decrypt = function (text) {
    var dec = this.decipher.update(text, 'hex', 'utf8');
    dec += this.decipher.final('utf8');
    return dec;
  };

  this.match = function (token, text) {
    return this.decrypt(token) === text;
  };

  this.cipher = _crypto2.default.createCipher('aes-256-cbc', secret);
  this.decipher = _crypto2.default.createDecipher('aes-256-cbc', secret);
};

exports.Tokenizer = Tokenizer;

var genToken = exports.genToken = function (secret) {
  return new Tokenizer(secret);
};
});

unwrapExports(tokenizer);
var tokenizer_1 = tokenizer.genToken;
var tokenizer_2 = tokenizer.Tokenizer;

var helpers = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Utils = undefined;



var _moment2 = _interopRequireDefault(moment);



function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var Utils = function Utils () {};

exports.Utils = Utils;

Utils.isFunction = function (val) { return typeof val === 'function'; };

Utils.isObject = function (val) { return typeof val === 'object'; };

Utils.genToken = tokenizer.genToken;

Utils.addDays = function (offset, date) {
  date = date || new Date();
  var momentAt = (0, _moment2.default)(date).add(offset, 'days');
  return momentAt.toDate();
};

Utils.isAfter = function (first, second) {
  var firstMoment = (0, _moment2.default)(first);
  var secondMoment = (0, _moment2.default)(second);
  return secondMoment.isAfter(firstMoment);
};
});

unwrapExports(helpers);
var helpers_1 = helpers.Utils;

var authenticable_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateMiddlerware = exports.saveMiddlerware = exports.hashedPassword = undefined;
exports.authenticable = authenticable;



var _assertPlus2 = _interopRequireDefault(_assertPlus);



var _mongoose2 = _interopRequireDefault(mongoose);



var _bcryptjs2 = _interopRequireDefault(bcryptjs);





function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var ref = helpers.Utils;
var isObject = ref.isObject;
var deviseError = new errors.DeviseError();
deviseError.code = 'EAUTH';

var hashedPassword = exports.hashedPassword = function (obj, next) {
  _bcryptjs2.default.hash(obj[options.passwordField], 10).then(function (hashed) {
    obj[options.passwordField] = hashed;
    next();
  }).catch(next);
};

var saveMiddlerware = exports.saveMiddlerware = function (next) {
  var self = this;

  if (!self.isModified(options.passwordField)) {
    next();
  } else {
    hashedPassword(self, next);
  }
};

var updateMiddlerware = exports.updateMiddlerware = function (next) {
  if (!this.getUpdate()[options.passwordField]) {
    next();
  } else {
    hashedPassword(this.getUpdate(), next);
  }
};

var options = {};

function authenticable(schema, opts) {
  _assertPlus2.default.func(schema.methods.t, 'translator method');

  _assertPlus2.default.func(schema.statics.t, 'translator method');

  options = opts || {};
  options.authenticationField = options.authenticationField || 'email';
  options.authenticationFieldMessage = options.authenticationFieldMessage || undefined;
  options.authenticationFieldType = options.authenticationFieldType || _mongoose2.default.Schema.Types.Email;
  options.passwordField = options.passwordField || 'password';
  options.authenticatorError = options.authenticatorError || ("No " + (options.authenticationField) + " provided");
  options.passwordError = options.passwordError || 'No password provided';
  options.authenticatorNotExistError = options.authenticatorNotExistError || ("Incorrect " + (options.authenticationField));
  options.credentialsNotExistError = options.credentialsNotExistError || 'Incorrect credentials';
  options.lockable.maximumAllowedFailedAttempts = options.lockable ? options.lockable.maximumAllowedFailedAttempts || 3 : 3;
  var authenticationFields = {};
  authenticationFields[options.authenticationField] = {
    type: options.authenticationFieldType,
    message: options.authenticationFieldMessage,
    unique: true,
    trim: true,
    index: true,
    required: true
  };
  authenticationFields[options.passwordField] = {
    type: String,
    required: true,
    select: false
  };
  schema.add(authenticationFields);
  schema.pre('save', saveMiddlerware);
  schema.pre('findOneAndUpdate', updateMiddlerware);
  schema.pre('update', updateMiddlerware);
  schema.pre('updateOne', updateMiddlerware);
  schema.pre('updateMany', updateMiddlerware);

  schema.methods.validPassword = function (password) {
    var self = this;

    if (!password) {
      return false;
    }

    if (!self[options.passwordField]) {
      return false;
    }

    return _bcryptjs2.default.compareSync(password, self[options.passwordField]);
  };

  schema.methods.authenticate = function (password, opts) {
    var self = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var passwordIsValid;

      if (!self.isConfirmed()) {
        var error$1;
        error$1 = self.throwConfirmedError();
        return $return(reject(error$1));
      }

      if (self.isLocked()) {
        var error$2;
        error$2 = self.throwLockedError();
        return $return(reject(error$2));
      }

      passwordIsValid = self.validPassword(password);

      if (passwordIsValid) {
        self.failedAttempts = 0;
        return Promise.resolve(self.track()).then(function ($await_6) {
          try {
            return $If_4.call(this);
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }.bind(this), $error);
      } else {
        var failedAttemptsExceed;
        self.failedAttempts = self.failedAttempts + 1;
        failedAttemptsExceed = self.failedAttempts >= options.lockable.maximumAllowedFailedAttempts;

        if (failedAttemptsExceed) {
          var $Try_1_Post = function () {
            try {
              return $If_5.call(this);
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }.bind(this);

          var $Try_1_Catch = function (error) {
            try {
              return $return(reject(error));
            } catch ($boundEx) {
              return $error($boundEx);
            }
          };

          try {
            return Promise.resolve(self.lock(opts)).then(function ($await_7) {
              try {
                return $return(resolve(passwordIsValid));
              } catch ($boundEx) {
                return $Try_1_Catch($boundEx);
              }
            }, $Try_1_Catch);
          } catch (error) {
            $Try_1_Catch(error);
          }
        }

        function $If_5() {
          return $If_4.call(this);
        }

        return $If_5.call(this);
      }

      function $If_4() {
        var $Try_2_Post = function () {
          try {
            return $return();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        var $Try_2_Catch = function (error) {
          try {
            reject(error);
            return $Try_2_Post();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        try {
          return Promise.resolve(self.save()).then(function ($await_8) {
            try {
              resolve(passwordIsValid);
              return $Try_2_Post();
            } catch ($boundEx) {
              return $Try_2_Catch($boundEx);
            }
          }, $Try_2_Catch);
        } catch (error) {
          $Try_2_Catch(error);
        }
      }
    }); });
  };

  schema.statics.authenticate = function (credentials, opts) {
    var this$1 = this;

    var Authenticable = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var criteria, authenticable;

      if (!isObject(credentials)) {
        deviseError.message = this.t('credentialsNotExistError');
        return $return(reject(deviseError));
      }

      if (!credentials[options.passwordField]) {
        deviseError.message = this.t('passwordError');
        return $return(reject(deviseError));
      }

      if (!credentials[options.authenticationField]) {
        deviseError.message = this.t('authenticatorError', {
          field: options.authenticationField
        });
        return $return(reject(deviseError));
      }

      criteria = {
        unregisteredAt: null
      };
      criteria[options.authenticationField] = credentials[options.authenticationField];
      return Promise.resolve(Authenticable.findOne(criteria, ("+" + (options.passwordField)))).then(function ($await_9) {
        try {
          authenticable = $await_9;

          if (!authenticable) {
            deviseError.message = this.t('authenticatorNotExistError', {
              field: options.authenticationField
            });
            return $return(reject(deviseError));
          }

          var $Try_3_Post = function () {
            try {
              return $return();
            } catch ($boundEx) {
              return $error($boundEx);
            }
          };

          var $Try_3_Catch = function (error) {
            try {
              reject(error);
              return $Try_3_Post();
            } catch ($boundEx) {
              return $error($boundEx);
            }
          };

          try {
            var res;
            return Promise.resolve(authenticable.authenticate(credentials[options.passwordField], opts)).then(function ($await_10) {
              try {
                res = $await_10;
                resolve(res ? authenticable : false);
                return $Try_3_Post();
              } catch ($boundEx) {
                return $Try_3_Catch($boundEx);
              }
            }, $Try_3_Catch);
          } catch (error) {
            $Try_3_Catch(error);
          }
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this$1)); });
  };
}
});

unwrapExports(authenticable_1);
var authenticable_2 = authenticable_1.updateMiddlerware;
var authenticable_3 = authenticable_1.saveMiddlerware;
var authenticable_4 = authenticable_1.hashedPassword;
var authenticable_5 = authenticable_1.authenticable;

var confirmable_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.confirmable = confirmable;



var _assertPlus2 = _interopRequireDefault(_assertPlus);





function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var ref = helpers.Utils;
var genToken = ref.genToken;
var addDays = ref.addDays;
var isAfter = ref.isAfter;
var isFunction = ref.isFunction;
var deviseError = new errors.DeviseError();
deviseError.code = 'ECONFIRM';
var options = {};

function confirmable(schema, opts) {
  _assertPlus2.default.func(schema.methods.sendNotification, 'sendNotification method');

  _assertPlus2.default.func(schema.methods.t, 'translator method');

  _assertPlus2.default.func(schema.statics.t, 'translator method');

  options = opts || {};
  options.authenticationField = options.authenticationField || 'email';
  options.invalidConfirmationTokenError = options.invalidConfirmationTokenError || 'Invalid confirmation token';
  options.confirmationTokenExpiredError = options.confirmationTokenExpiredError || 'Confirmation token expired';
  options.accountNotConfirmedError = options.accountNotConfirmedError || 'Account not confirmed';
  options.confirmable.tokenLifeSpan = options.confirmable ? options.confirmable.tokenLifeSpan || 3 : 3;
  schema.add({
    confirmationToken: {
      type: String,
      default: null,
      index: true
    },
    confirmationTokenExpiryAt: {
      type: Date,
      default: null,
      index: true
    },
    confirmedAt: {
      type: Date,
      default: null,
      index: true
    },
    confirmationSentAt: {
      type: Date,
      default: null,
      index: true
    }
  });

  schema.methods.throwConfirmedError = function () {
    deviseError.message = this.t('accountNotConfirmedError');
    return deviseError;
  };

  schema.methods.hasConfirmationTokenExpired = function () {
    return !isAfter(new Date(), this.confirmationTokenExpiryAt);
  };

  schema.methods.isConfirmed = function () {
    return this.confirmedAt !== null;
  };

  schema.methods.generateConfirmationToken = function () {
    var self = this;
    self.confirmationTokenExpiryAt = addDays(options.confirmable.tokenLifeSpan);
    var tokenizer = genToken(self.confirmationTokenExpiryAt.getTime().toString());
    self.confirmationToken = tokenizer.encrypt(self[options.authenticationField]);
    self.confirmedAt = null;
  };

  schema.methods.sendConfirmation = function (opts) {
    var self = this;
    self.sendNotification(self, 'account_confirmation', function (fn) { return new Promise(function ($return, $error) {
      var $Try_1_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_1_Catch = function (error) {
        try {
          console.log(("[Error]: sendConfirmation - " + (error.stack)));
          return $Try_1_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        self.confirmationSentAt = new Date();

        if (isFunction(fn)) {
          return Promise.resolve(fn(opts)).then(function ($await_5) {
            try {
              return $If_4.call(this);
            } catch ($boundEx) {
              return $Try_1_Catch($boundEx);
            }
          }.bind(this), $Try_1_Catch);
        }

        function $If_4() {
          return $Try_1_Post();
        }

        return $If_4.call(this);
      } catch (error) {
        $Try_1_Catch(error);
      }
    }); });
  };

  schema.methods.confirm = function () {
    var this$1 = this;

    var self = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var val, tokenizer, confirmed;

      if (!self.confirmationToken) {
        deviseError.message = this.t('invalidConfirmationTokenError');
        return $return(reject(deviseError));
      }

      if (this.hasConfirmationTokenExpired()) {
        deviseError.message = this.t('confirmationTokenExpiredError');
        return $return(reject(deviseError));
      }

      val = self.confirmationTokenExpiryAt.getTime().toString();
      tokenizer = genToken(val);
      confirmed = tokenizer.match(self.confirmationToken, self[options.authenticationField]);

      if (!confirmed) {
        deviseError.message = this.t('invalidConfirmationTokenError');
        return $return(reject(deviseError));
      }

      self.confirmedAt = new Date();

      var $Try_2_Post = function () {
        try {
          resolve(confirmed);
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_2_Catch = function (error) {
        try {
          return $return(reject(error));
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        return Promise.resolve(self.save()).then(function ($await_6) {
          try {
            return $Try_2_Post();
          } catch ($boundEx) {
            return $Try_2_Catch($boundEx);
          }
        }, $Try_2_Catch);
      } catch (error) {
        $Try_2_Catch(error);
      }
    }.bind(this$1)); });
  };

  schema.statics.confirm = function (confirmationToken) {
    var this$1 = this;

    var Confirmable = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var confirmable;
      return Promise.resolve(Confirmable.findOne({
        confirmationToken: confirmationToken
      })).then(function ($await_7) {
        try {
          confirmable = $await_7;

          if (!confirmable) {
            deviseError.message = this.t('invalidConfirmationTokenError');
            return $return(reject(deviseError));
          }

          var $Try_3_Post = function () {
            try {
              resolve(confirmable);
              return $return();
            } catch ($boundEx) {
              return $error($boundEx);
            }
          };

          var $Try_3_Catch = function (error) {
            try {
              return $return(reject(error));
            } catch ($boundEx) {
              return $error($boundEx);
            }
          };

          try {
            return Promise.resolve(confirmable.confirm()).then(function ($await_8) {
              try {
                return $Try_3_Post();
              } catch ($boundEx) {
                return $Try_3_Catch($boundEx);
              }
            }, $Try_3_Catch);
          } catch (error) {
            $Try_3_Catch(error);
          }
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this$1)); });
  };
}
});

unwrapExports(confirmable_1);
var confirmable_2 = confirmable_1.confirmable;

var lockable_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.lockable = lockable;



var _assertPlus2 = _interopRequireDefault(_assertPlus);





function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var ref = helpers.Utils;
var genToken = ref.genToken;
var addDays = ref.addDays;
var isFunction = ref.isFunction;
var isAfter = ref.isAfter;
var deviseError = new errors.DeviseError();
deviseError.code = 'ELOCK';
var options = {};

function lockable(schema, opts) {
  _assertPlus2.default.func(schema.methods.sendNotification, 'sendNotification method');

  _assertPlus2.default.func(schema.methods.t, 'translator method');

  _assertPlus2.default.func(schema.statics.t, 'translator method');

  options = opts || {};
  options.authenticationField = options.authenticationField || 'email';
  options.accountLockedError = options.accountLockedError || 'Account locked. Check unlock instructions sent to you.';
  options.invalidUnlockTokenError = options.invalidUnlockTokenError || 'Invalid unlock token';
  options.unlockTokenExpiredError = options.unlockTokenExpiredError || 'Unlock token expired';
  options.lockable.tokenLifeSpan = options.lockable ? options.lockable.tokenLifeSpan || 3 : 3;
  schema.add({
    failedAttempts: {
      type: Number,
      default: 0,
      index: true
    },
    lockedAt: {
      type: Date,
      default: null,
      index: true
    },
    unlockedAt: {
      type: Date,
      default: null,
      index: true
    },
    unlockToken: {
      type: String,
      default: null,
      index: true
    },
    unlockTokenSentAt: {
      type: Date,
      default: null,
      index: true
    },
    unlockTokenExpiryAt: {
      type: Date,
      default: null,
      index: true
    }
  });

  schema.methods.throwLockedError = function () {
    deviseError.message = this.t('accountLockedError');
    return deviseError;
  };

  schema.methods.hasUnlockTokenExpired = function () {
    return !isAfter(new Date(), this.unlockTokenExpiryAt);
  };

  schema.methods.isLocked = function () {
    return this.lockedAt !== null;
  };

  schema.methods.generateUnlockToken = function () {
    var self = this;
    self.unlockTokenExpiryAt = addDays(options.lockable.tokenLifeSpan);
    var tokenizer = genToken(self.unlockTokenExpiryAt.getTime().toString());
    self.unlockToken = tokenizer.encrypt(self[options.authenticationField]);
    self.unlockedAt = null;
  };

  schema.methods.sendUnlock = function (opts) {
    var self = this;
    self.sendNotification(self, 'account_recovery', function (fn) { return new Promise(function ($return, $error) {
      var $Try_1_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_1_Catch = function (error) {
        try {
          console.log(("[Error]: sendUnlock - " + (error.stack)));
          return $Try_1_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        self.unlockTokenSentAt = new Date();

        if (isFunction(fn)) {
          return Promise.resolve(fn(opts)).then(function ($await_5) {
            try {
              return $If_4.call(this);
            } catch ($boundEx) {
              return $Try_1_Catch($boundEx);
            }
          }.bind(this), $Try_1_Catch);
        }

        function $If_4() {
          return $Try_1_Post();
        }

        return $If_4.call(this);
      } catch (error) {
        $Try_1_Catch(error);
      }
    }); });
  };

  schema.methods.lock = function (opts) {
    var self = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var $Try_2_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_2_Catch = function (error) {
        try {
          reject(error);
          return $Try_2_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        self.lockedAt = new Date();
        return Promise.resolve(self.generateUnlockToken()).then(function ($await_6) {
          try {
            return Promise.resolve(self.sendUnlock(opts)).then(function ($await_7) {
              try {
                return Promise.resolve(self.save()).then(function ($await_8) {
                  try {
                    resolve(true);
                    return $Try_2_Post();
                  } catch ($boundEx) {
                    return $Try_2_Catch($boundEx);
                  }
                }, $Try_2_Catch);
              } catch ($boundEx) {
                return $Try_2_Catch($boundEx);
              }
            }, $Try_2_Catch);
          } catch ($boundEx) {
            return $Try_2_Catch($boundEx);
          }
        }, $Try_2_Catch);
      } catch (error) {
        $Try_2_Catch(error);
      }
    }); });
  };

  schema.statics.unlock = function (unlockToken) {
    var this$1 = this;

    var Lockable = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var $Try_3_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_3_Catch = function (error) {
        try {
          reject(error);
          return $Try_3_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        var lockable, val, tokenizer;
        return Promise.resolve(Lockable.findOne({
          unlockToken: unlockToken
        })).then(function ($await_9) {
          try {
            lockable = $await_9;

            if (!lockable) {
              deviseError.message = this.t('invalidUnlockTokenError');
              return $return(reject(deviseError));
            }

            if (lockable.hasUnlockTokenExpired()) {
              deviseError.message = this.t('unlockTokenExpiredError');
              return $return(reject(deviseError));
            }

            val = lockable.unlockTokenExpiryAt.getTime().toString();
            tokenizer = genToken(val);

            if (!tokenizer.match(unlockToken, lockable[options.authenticationField])) {
              deviseError.message = this.t('invalidUnlockTokenError');
              return $return(reject(deviseError));
            }

            lockable.unlockedAt = new Date();
            lockable.failedAttempts = 0;
            lockable.lockedAt = null;
            return Promise.resolve(lockable.save()).then(function ($await_10) {
              try {
                resolve(lockable);
                return $Try_3_Post();
              } catch ($boundEx) {
                return $Try_3_Catch($boundEx);
              }
            }, $Try_3_Catch);
          } catch ($boundEx) {
            return $Try_3_Catch($boundEx);
          }
        }.bind(this), $Try_3_Catch);
      } catch (error) {
        $Try_3_Catch(error);
      }
    }.bind(this$1)); });
  };
}
});

unwrapExports(lockable_1);
var lockable_2 = lockable_1.lockable;

var recoverable_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.recoverable = recoverable;



var _assertPlus2 = _interopRequireDefault(_assertPlus);





function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var ref = helpers.Utils;
var genToken = ref.genToken;
var addDays = ref.addDays;
var isFunction = ref.isFunction;
var isAfter = ref.isAfter;
var isObject = ref.isObject;
var deviseError = new errors.DeviseError();
deviseError.code = 'ERECOV';
var options = {};

function recoverable(schema, opts) {
  _assertPlus2.default.func(schema.methods.sendNotification, 'sendNotification method');

  _assertPlus2.default.func(schema.methods.t, 'translator method');

  _assertPlus2.default.func(schema.statics.t, 'translator method');

  options = opts || {};
  options.authenticationField = options.authenticationField || 'email';
  options.passwordField = options.passwordField || 'password';
  options.accountWithoutAssociationError = options.accountWithoutAssociationError || ("Account without association with " + (options.authenticationField));
  options.invalidRecoveryTokenError = options.invalidRecoveryTokenError || 'Invalid recovery token';
  options.recoveryTokenExpiredError = options.recoveryTokenExpiredError || 'Recovery token expired';
  options.authenticatorError = options.authenticatorError || ("No " + (options.authenticationField) + " provided");
  options.credentialsNotExistError = options.credentialsNotExistError || 'Incorrect credentials';
  options.recoverable.tokenLifeSpan = options.recoverable ? options.recoverable.tokenLifeSpan || 3 : 3;
  schema.add({
    recoveryToken: {
      type: String,
      default: null,
      index: true
    },
    recoveryTokenExpiryAt: {
      type: Date,
      default: null,
      index: true
    },
    recoverySentAt: {
      type: Date,
      default: null,
      index: true
    },
    recoveredAt: {
      type: Date,
      default: null,
      index: true
    }
  });

  schema.methods.hasRecoveryTokenExpired = function () {
    return !isAfter(new Date(), this.recoveryTokenExpiryAt);
  };

  schema.methods.generateRecoveryToken = function () {
    var self = this;
    self.recoveryTokenExpiryAt = addDays(options.recoverable.tokenLifeSpan);
    var tokenizer = genToken(self.recoveryTokenExpiryAt.getTime().toString());
    self.recoveryToken = tokenizer.encrypt(self[options.authenticationField]);
    self.recoveredAt = null;
  };

  schema.methods.sendRecovery = function (opts) {
    var self = this;
    self.sendNotification(self, 'password_recovery', function (fn) { return new Promise(function ($return, $error) {
      var $Try_1_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_1_Catch = function (error) {
        try {
          console.log(("[Error]: sendRecovery - " + (error.stack)));
          return $Try_1_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        self.recoverySentAt = new Date();

        if (isFunction(fn)) {
          return Promise.resolve(fn(opts)).then(function ($await_5) {
            try {
              return $If_4.call(this);
            } catch ($boundEx) {
              return $Try_1_Catch($boundEx);
            }
          }.bind(this), $Try_1_Catch);
        }

        function $If_4() {
          return $Try_1_Post();
        }

        return $If_4.call(this);
      } catch (error) {
        $Try_1_Catch(error);
      }
    }); });
  };

  schema.statics.requestRecover = function (credentials, opts) {
    var this$1 = this;

    var Recoverable = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var criteria;

      if (!isObject(credentials) || !credentials[options.authenticationField]) {
        deviseError.message = this.t('authenticatorError', {
          field: options.authenticationField
        });
        return $return(reject(deviseError));
      }

      criteria = {};
      criteria[options.authenticationField] = credentials[options.authenticationField];

      var $Try_2_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_2_Catch = function (error) {
        try {
          reject(error);
          return $Try_2_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        var recoverable;
        return Promise.resolve(Recoverable.findOne(criteria)).then(function ($await_6) {
          try {
            recoverable = $await_6;

            if (!recoverable) {
              deviseError.message = this.t('accountWithoutAssociationError');
              return $return(reject(deviseError));
            }

            return Promise.resolve(recoverable.generateRecoveryToken()).then(function ($await_7) {
              try {
                return Promise.resolve(recoverable.sendRecovery(opts)).then(function ($await_8) {
                  try {
                    return Promise.resolve(recoverable.save()).then(function ($await_9) {
                      try {
                        resolve(recoverable);
                        return $Try_2_Post();
                      } catch ($boundEx) {
                        return $Try_2_Catch($boundEx);
                      }
                    }, $Try_2_Catch);
                  } catch ($boundEx) {
                    return $Try_2_Catch($boundEx);
                  }
                }, $Try_2_Catch);
              } catch ($boundEx) {
                return $Try_2_Catch($boundEx);
              }
            }, $Try_2_Catch);
          } catch ($boundEx) {
            return $Try_2_Catch($boundEx);
          }
        }.bind(this), $Try_2_Catch);
      } catch (error) {
        $Try_2_Catch(error);
      }
    }.bind(this$1)); });
  };

  schema.statics.recover = function (recoveryToken, newPassword) {
    var this$1 = this;

    var Recoverable = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      var $Try_3_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_3_Catch = function (error) {
        try {
          reject(error);
          return $Try_3_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        var recoverable, val, tokenizer;
        return Promise.resolve(Recoverable.findOne({
          recoveryToken: recoveryToken
        })).then(function ($await_10) {
          try {
            recoverable = $await_10;

            if (!recoverable) {
              deviseError.message = this.t('invalidRecoveryTokenError');
              return $return(reject(deviseError));
            }

            if (recoverable.hasRecoveryTokenExpired()) {
              deviseError.message = this.t('recoveryTokenExpiredError');
              return $return(reject(deviseError));
            }

            val = recoverable.recoveryTokenExpiryAt.getTime().toString();
            tokenizer = genToken(val);

            if (!tokenizer.match(recoveryToken, recoverable[options.authenticationField])) {
              deviseError.message = this.t('invalidRecoveryTokenError');
              return $return(reject(deviseError));
            }

            recoverable[options.passwordField] = newPassword;
            recoverable.recoveredAt = new Date();
            recoverable.recoveryToken = null;
            return Promise.resolve(recoverable.save()).then(function ($await_11) {
              try {
                resolve(recoverable);
                return $Try_3_Post();
              } catch ($boundEx) {
                return $Try_3_Catch($boundEx);
              }
            }, $Try_3_Catch);
          } catch ($boundEx) {
            return $Try_3_Catch($boundEx);
          }
        }.bind(this), $Try_3_Catch);
      } catch (error) {
        $Try_3_Catch(error);
      }
    }.bind(this$1)); });
  };
}
});

unwrapExports(recoverable_1);
var recoverable_2 = recoverable_1.recoverable;

var registerable_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerable = registerable;



var _assertPlus2 = _interopRequireDefault(_assertPlus);





function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

var ref = helpers.Utils;
var isFunction = ref.isFunction;
var isObject = ref.isObject;
var deviseError = new errors.DeviseError();
deviseError.code = 'EREGIST';
var options = {};

function registerable(schema, opts) {
  _assertPlus2.default.func(schema.methods.t, 'translator method');

  _assertPlus2.default.func(schema.statics.t, 'translator method');

  options = opts || {};
  options.authenticationField = options.authenticationField || 'email';
  options.definitionsNotFoundError = options.definitionsNotFoundError || 'Definitions not found';
  schema.add({
    registeredAt: {
      type: Date,
      default: null,
      index: true
    },
    unregisteredAt: {
      type: Date,
      default: null,
      index: true
    }
  });

  schema.methods.unregister = function (beforeUnregister, afterUnregister) {
    return new Promise(function ($return, $error) {
      var self = this;
      return $return(new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
        var $Try_1_Post = function () {
          try {
            return $return();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        var $Try_1_Catch = function (error) {
          try {
            reject(error);
            return $Try_1_Post();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        try {
          if (isFunction(beforeUnregister)) {
            return Promise.resolve(beforeUnregister(self)).then(function ($await_6) {
              try {
                return $If_4.call(this);
              } catch ($boundEx) {
                return $Try_1_Catch($boundEx);
              }
            }.bind(this), $Try_1_Catch);
          }

          function $If_4() {
            self.unregisteredAt = new Date();
            return Promise.resolve(self.save()).then(function ($await_7) {
              try {
                if (isFunction(afterUnregister)) {
                  return Promise.resolve(afterUnregister(self)).then(function ($await_8) {
                    try {
                      return $If_5.call(this);
                    } catch ($boundEx) {
                      return $Try_1_Catch($boundEx);
                    }
                  }.bind(this), $Try_1_Catch);
                }

                function $If_5() {
                  resolve(true);
                  return $Try_1_Post();
                }

                return $If_5.call(this);
              } catch ($boundEx) {
                return $Try_1_Catch($boundEx);
              }
            }.bind(this), $Try_1_Catch);
          }

          return $If_4.call(this);
        } catch (error) {
          $Try_1_Catch(error);
        }
      }); }));
    }.bind(this));
  };

  schema.methods.register = function (opts) {
    return new Promise(function ($return, $error) {
      var self = this;
      return $return(new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
        return Promise.resolve(self.generateConfirmationToken()).then(function ($await_9) {
          try {
            self.registeredAt = new Date();
            self.confirmationSentAt = new Date();

            var $Try_2_Post = function () {
              try {
                return $return();
              } catch ($boundEx) {
                return $error($boundEx);
              }
            };

            var $Try_2_Catch = function (error) {
              try {
                reject(error);
                return $Try_2_Post();
              } catch ($boundEx) {
                return $error($boundEx);
              }
            };

            try {
              return Promise.resolve(self.save()).then(function ($await_10) {
                try {
                  return Promise.resolve(self.sendConfirmation(opts)).then(function ($await_11) {
                    try {
                      resolve(true);
                      return $Try_2_Post();
                    } catch ($boundEx) {
                      return $Try_2_Catch($boundEx);
                    }
                  }, $Try_2_Catch);
                } catch ($boundEx) {
                  return $Try_2_Catch($boundEx);
                }
              }, $Try_2_Catch);
            } catch (error) {
              $Try_2_Catch(error);
            }
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }, $error);
      }); }));
    }.bind(this));
  };

  schema.statics.register = function (definitions, opts) {
    var this$1 = this;

    var Registerable = this;
    return new Promise(function (resolve, reject) { return new Promise(function ($return, $error) {
      if (!isObject(definitions)) {
        deviseError.message = this.t('definitionsNotFoundError');
        return $return(reject(deviseError));
      }

      var $Try_3_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_3_Catch = function (error) {
        try {
          reject(error);
          return $Try_3_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        var registerable;
        registerable = new Registerable(definitions);
        return Promise.resolve(registerable.register(opts)).then(function ($await_12) {
          try {
            resolve(registerable);
            return $Try_3_Post();
          } catch ($boundEx) {
            return $Try_3_Catch($boundEx);
          }
        }, $Try_3_Catch);
      } catch (error) {
        $Try_3_Catch(error);
      }
    }.bind(this$1)); });
  };
}
});

unwrapExports(registerable_1);
var registerable_2 = registerable_1.registerable;

var trackable_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.trackable = trackable;



function externalIpAddress() {
  var ref;

  try {
    return (ref = []).concat.apply(ref, Object.values((0, os.networkInterfaces)())).filter(function (details) { return details.family === 'IPv4' && !details.internal; }).pop().address;
  } catch (_e) {
    return '';
  }
}

function trackable(schema) {
  schema.add({
    signInCount: {
      type: Number,
      default: 0,
      index: true
    },
    currentSignInAt: {
      type: Date,
      default: null,
      index: true
    },
    currentSignInIpAddress: {
      type: String,
      default: null,
      index: true
    },
    lastSignInAt: {
      type: Date,
      default: null,
      index: true
    },
    lastSignInIpAddress: {
      type: String,
      default: null,
      index: true
    }
  });

  schema.methods.track = function (ipAddress) {
    return new Promise(function ($return, $error) {
      var self;
      self = this;
      self.signInCount = self.signInCount + 1;
      self.lastSignInAt = self.currentSignInAt;
      self.lastSignInIpAddress = self.currentSignInIpAddress;
      self.currentSignInAt = new Date();
      return Promise.resolve(new Promise(function ($return, $error) {
        var $logicalOr_2;

        if (!($logicalOr_2 = ipAddress)) {
          return Promise.resolve(externalIpAddress()).then(function ($await_4) {
            try {
              $logicalOr_2 = $await_4;
              return $If_3.call(this);
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }.bind(this), $error);
        }

        function $If_3() {
          return $return($logicalOr_2);
        }

        return $If_3.call(this);
      })).then(function ($await_5) {
        try {
          self.currentSignInIpAddress = $await_5;
          return Promise.resolve(self.save()).then(function ($await_6) {
            try {
              return $return();
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }, $error);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }, $error);
    }.bind(this));
  };
}
});

unwrapExports(trackable_1);
var trackable_2 = trackable_1.trackable;

var build = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.devise = devise;















var options = {};

function devise(schema, opts) {
  options = Object.assign({
    confirmable: {
      tokenLifeSpan: 3
    },
    lockable: {
      tokenLifeSpan: 3,
      maximumAllowedFailedAttempts: 3
    },
    recoverable: {
      tokenLifeSpan: 3
    },
    registerable: {
      autoConfirm: false
    }
  }, opts);

  function stringFormat(key, values) {
    var message = options[key];

    if (values) {
      Object.keys(values).forEach(function (param) {
        message = message.replace(("{{" + param + "}}"), values[param]);
      });
    }

    return message;
  }

  function t(key, values) {
    return options.i18n ? options.i18n.t(key, values) : stringFormat(key, values);
  }

  function sendNotification(record, action, done) {
    done();
  }

  schema.methods.sendNotification = schema.methods.send || sendNotification;
  schema.methods.t = schema.methods.t || t;
  schema.statics.t = schema.statics.t || t;
  (0, authenticable_1.authenticable)(schema, options);
  (0, confirmable_1.confirmable)(schema, options);
  (0, lockable_1.lockable)(schema, options);
  (0, recoverable_1.recoverable)(schema, options);
  (0, registerable_1.registerable)(schema, options);
  (0, trackable_1.trackable)(schema);
}

exports.default = devise;
});

var index$2 = unwrapExports(build);
var build_1 = build.devise;

export default index$2;
export { build_1 as devise };
